use common::Achievement;
use common::ToggleAchievement;
use reqwasm::http::Request;
use wasm_bindgen_futures::spawn_local;
use yew::function_component;
use yew::html;
use yew::Callback;
use yew::Html;
use yew::Properties;

#[derive(Properties, PartialEq)]
pub struct Props {
    pub achievement: Achievement,
}

#[function_component]
pub fn AchievementComponent(props: &Props) -> Html {
    let Achievement {
        goal,
        completed,
        uuid,
    } = &props.achievement;

    let uuid = *uuid;

    let onclick = Callback::from(move |_| {
        log::info!("button click, toggling achievement");

        let payload = ToggleAchievement { uuid };
        let payload = serde_json::to_string(&payload).unwrap();

        spawn_local(async move {
            let req = Request::post("http://127.0.0.1:4000/toggle")
                .header("Content-Type", "application/json")
                .body(payload);
            let res = req.send().await;
            match res {
                Ok(response) => {
                    dbg!(response);
                }
                Err(err) => {
                    log::error!("Request failed: {err:?}");
                }
            }
        });
    });

    let button_class = if *completed { "button-primary" } else { "button" };

    html! {
        <div class="row">
            <div class="nine columns">
                <p>{goal}</p>
            </div>
            <div class="three columns">
                <button {onclick} class={button_class}>{"✓"}</button>
            </div>
        </div>
    }
}
