use crate::components::achievement::AchievementComponent;
use crate::event_bus::EventBus;
use crate::services::websocket::WebsocketService;
use std::rc::Rc;
use yew::prelude::*;
use yew_agent::Bridge;
use yew_agent::Bridged;

pub struct Root {
    wss: WebsocketService,
    _producer: Box<dyn Bridge<EventBus>>,

    /// None until the first websocket message is received.
    state: Option<common::State>,
}

pub enum Msg {
    HandleMsg(String),
}

impl Component for Root {
    type Message = Msg;
    type Properties = ();

    fn create(ctx: &Context<Self>) -> Self {
        let wss = WebsocketService::new();

        let cb = {
            let link = ctx.link().clone();
            move |e| link.send_message(Msg::HandleMsg(e))
        };

        Self {
            wss,
            _producer: EventBus::bridge(Rc::new(cb)),
            state: None,
        }
    }

    fn update(&mut self, _ctx: &Context<Self>, msg: Self::Message) -> bool {
        match msg {
            Msg::HandleMsg(s) => {
                let msg: common::WebSocketMessage = serde_json::from_str(&s).unwrap();
                self.state = Some(msg);
                true
            }
        }
    }

    fn view(&self, ctx: &Context<Self>) -> Html {
        let Some(state) = &self.state else {
            return html! {
                <p>{"loading..."}</p>
            };
        };

        let achievements = state
            .achievements
            .iter()
            .cloned()
            .map(|a| {
                html! {
                    <AchievementComponent achievement={a} />
                }
            })
            .collect::<Html>();

        html! {
            <div class="container">
              <div class="row">
                <div class="twelve columns" style="margin-top: 25%">
                    <h4>{"Achievements"}</h4>
                </div>
              </div>

              {achievements}
            </div>
        }
    }
}
