use serde::Deserialize;
use serde::Serialize;
use std::collections::HashSet;
use yew_agent::HandlerId;
use yew_agent::Public;
use yew_agent::WorkerLink;

pub struct EventBus {
    link: WorkerLink<Self>,
    subscribers: HashSet<HandlerId>,
}

#[derive(Serialize, Deserialize)]
pub enum EventBusInput {
    EventBusMsg(String),
}

impl yew_agent::Worker for EventBus {
    type Reach = Public<Self>;
    type Input = EventBusInput;
    type Output = String;
    type Message = ();

    fn create(link: WorkerLink<Self>) -> Self {
        Self {
            link,
            subscribers: HashSet::new(),
        }
    }

    fn update(&mut self, _msg: Self::Message) {}

    fn handle_input(&mut self, msg: Self::Input, id: HandlerId) {
        match msg {
            EventBusInput::EventBusMsg(s) => {
                for sub in &self.subscribers {
                    self.link.respond(*sub, s.clone());
                }
            }
        }
    }

    fn connected(&mut self, id: HandlerId) {
        self.subscribers.insert(id);
    }

    fn disconnected(&mut self, id: HandlerId) {
        self.subscribers.remove(&id);
    }

    fn name_of_resource() -> &'static str {
        "event_bus.js"
    }
}
